/*
 * client.c	spwauth client side functionality
 *
 * Copyright (C) 1996, Olaf Kirch <okir@monad.swb.de>
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <rpc/rpc.h>
#include <rpcsvc/ypclnt.h>
#include <memory.h>
#include "spwauth_prot.h"

#define SPWAUTH_PRIVATE
#include "spwauth.h"

static spwcookie *
spwauthproc_getcookie_1(spwuser *argp, CLIENT *clnt)
{
	static spwcookie	clnt_res;
	struct timeval		timeout = { 25, 0 };
	int			rpcres;

	memset(&clnt_res, 0, sizeof(clnt_res));
	rpcres = clnt_call(clnt, SPWAUTHPROC_GETCOOKIE,
				(xdrproc_t) xdr_spwuser, argp,
				(xdrproc_t) xdr_spwcookie, &clnt_res,
				timeout);
	return (rpcres == RPC_SUCCESS)? &clnt_res : NULL;
}

/*
 * Verify the password given by a user
 */
int
spwauth_check(char *user, char *passwd)
{
	spwcookie	*cookie;
	spwuser		spu;
	char		*domain, *master, *crypted;
	unsigned char	result[HASH_BYTES];
	CLIENT		*clnt;

#if 1
	if (!yp_get_default_domain(&domain)) {
		fprintf(stderr, "can't get NIS default domain\n");
		return SPWA_ERROR;
	}
	if (yp_master(domain, SHADOWMAP, &master) != 0) {
		fprintf(stderr, "no NIS master for %s\n", SHADOWMAP);
		return SPWA_ERROR;
	}
#else
	domain = "foobar";
	master = "localhost";
#endif
	if (!(clnt = clnt_create(master, SPWAUTHPROG, SPWAUTHVERS, "tcp"))) {
		clnt_pcreateerror("can't bind to spwauth server");
		return SPWA_ERROR;
	}

	spu.spw_domain = domain;
	spu.spw_name = user;
	if (!(cookie = spwauthproc_getcookie_1(&spu, clnt))) {
		clnt_perror(clnt, "can't contact server");
		clnt_destroy(clnt);
		return SPWA_ERROR;
	}
	clnt_destroy(clnt);

	crypted = spcrypt(passwd, cookie->spw_salt);
	compute_hash(result, cookie->spw_seed, crypted, strlen(crypted));
	if (memcmp(result, cookie->spw_hash, HASH_BYTES))
		return SPWA_FAIL;
	return SPWA_SUCCESS;
}

#ifdef TEST
int main(int argc, char **argv)
{
	char	*user, *pass;

	if (argc == 1) {
		user = getlogin();
	} else if (argc == 2) {
		user = argv[1];
	} else {
		fprintf(stderr, "usage: spwtest [user]\n");
		return 1;
	}

	pass = getpass("Please enter password:");
	switch (spwauth_check(user, pass)) {
	case SPWA_ERROR:
		fprintf(stderr, "Error during authentication. Aborting\n");
		return 1;
	case SPWA_FAIL:
		fprintf(stderr, "Invalid password. Sorry.\n");
		return 1;
	case SPWA_SUCCESS:
		fprintf(stderr, "Authentication passed...\n");
	}

	return 0;
}
#endif
