/*
 * spwauthd	Server process for SPWAUTH protocol
 *
 * Copyright (C) 1996, Olaf Kirch <okir@monad.swb.de>
 */

#include <stdio.h>
#include <stdlib.h>
#include <rpc/rpc.h>
#include <rpc/pmap_clnt.h>
#include <string.h>
#include <memory.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <syslog.h>
#include "spwauth_prot.h"

#define SPWAUTH_PRIVATE
#include "spwauth.h"

static char	saltchars[] =
			"abcdefghijklmnopqrstuvwxyz"
			"ABCDEFGHIJKLMNOPQRSTUVWXYZ"
			"0123456789";
#define NRSALTCH	(sizeof(saltchars)/sizeof(saltchars[0]))

typedef	void *	(*svcproc_t)(void *arg, struct svc_req *rqstp);

/*
 * Compute cookie
 */
static void *
spwauthproc_getcookie_1_svc(spwuser *spu, struct svc_req *rqstp)
{
	static spwcookie	ck;
	struct spwd		*spw;

	memset(&ck, 0, sizeof(ck));
	get_random(ck.spw_seed);

	/* Get shadow entry and compute cookie */
	if ((spw = splookup(spu->spw_domain, spu->spw_name)) != NULL) {
		memcpy(ck.spw_salt, spw->sp_pwdp, 2);
		compute_hash(ck.spw_hash, ck.spw_seed, spw->sp_pwdp,
						strlen(spw->sp_pwdp));
	} else {
		/* No user found, return random garbage */
		unsigned char		buf[64];

		get_random(buf);
		compute_hash(ck.spw_hash, ck.spw_seed, buf, HASH_BYTES);
		ck.spw_salt[0] = saltchars[buf[0] % NRSALTCH];
		ck.spw_salt[1] = saltchars[buf[1] % NRSALTCH];
	}

	return &ck;
}

/*
 * The RPC service function
 */
static void
spwauthprog_1(struct svc_req *rqstp, register SVCXPRT *transp)
{
	union {
		spwuser spwauthproc_getcookie_1_arg;
	} argument;
	xdrproc_t	xdr_argument, xdr_result;
	svcproc_t	svc_proc;
	void		*result;

	switch (rqstp->rq_proc) {
	case NULLPROC:
		svc_sendreply(transp, (xdrproc_t) xdr_void, (char *)NULL);
		return;

	case SPWAUTHPROC_GETCOOKIE:
		xdr_argument = (xdrproc_t) xdr_spwuser;
		xdr_result = (xdrproc_t) xdr_spwcookie;
		svc_proc = (svcproc_t) spwauthproc_getcookie_1_svc;
		break;

	default:
		svcerr_noproc(transp);
		return;
	}

	memset(&argument, 0, sizeof(argument));
	if (!svc_getargs(transp, xdr_argument, (caddr_t) &argument)) {
		svcerr_decode(transp);
		return;
	}
	result = svc_proc(&argument, rqstp);
	if (result != NULL && !svc_sendreply(transp, xdr_result, result)) {
		svcerr_systemerr(transp);
	}
	if (!svc_freeargs(transp, xdr_argument, (caddr_t) &argument)) {
		fprintf(stderr, "unable to free arguments");
		exit(1);
	}
	return;
}

int
main(int argc, char **argv)
{
	register SVCXPRT *transp;

	openlog("spwauthd", 0, LOG_DAEMON);

	pmap_unset(SPWAUTHPROG, SPWAUTHVERS);

	transp = svcudp_create(RPC_ANYSOCK);
	if (transp == NULL) {
		fprintf(stderr, "cannot create udp service.");
		exit(1);
	}
	if (!svc_register(transp, SPWAUTHPROG, SPWAUTHVERS, spwauthprog_1, IPPROTO_UDP)) {
		fprintf(stderr,
			"unable to register (SPWAUTHPROG, SPWAUTHVERS, udp).");
		exit(1);
	}

	transp = svctcp_create(RPC_ANYSOCK, 0, 0);
	if (transp == NULL) {
		fprintf(stderr, "cannot create tcp service.");
		exit(1);
	}
	if (!svc_register(transp, SPWAUTHPROG, SPWAUTHVERS, spwauthprog_1, IPPROTO_TCP)) {
		fprintf(stderr,
			"unable to register (SPWAUTHPROG, SPWAUTHVERS, tcp).");
		exit(1);
	}

	svc_run();
	fprintf(stderr, "svc_run returned");
	return 1;
}
